(define junior-backend "/net-junior")
(define avail-ifaces (woo-list/name+label "/net-junior"))
(define avail-masks (woo-list/name+label "/net-junior/eth0/avail_masks"))

;
; Common functions
;

; Constraint for interface widgets visibility (static/dhcp)
(define (iface_conf_type_changed)
  (iface_statical_conf visibility (iface_is_static))
  (iface_dhcp_conf visibility (iface_is_dhcp)))

; Global constraint for widgets visibility (manual/zeroconf)
(define (conf_type_changed)
  ((widgets interfaces_group dns_group) visibility (conf_type_is_manual))
  (and (conf_type_is_manual)
       (update-interface (current-interface))))

; Return #t if user select static configuration for current selected interface
(define (iface_is_static)
  (= (iface-conf current) 0))

; Return #t if user select dhcp configuration for current selected interface
(define (iface_is_dhcp)
  (= (iface-conf current) 1))

; Return #t if user select ipv4ll configuration for current selected interface
(define (iface_is_ip4ll)
  (= (iface-conf current) 2))

; return #t if global configuration is auto (zeroconf)
(define (conf_type_is_auto)
   (conf_auto state) )

; return #t if global configuration is manual (dhcp or static)
(define (conf_type_is_manual)
  (not (conf_type_is_auto)))

; Return current selected interface
(define (current-interface)
  (and (number? (ifaces current))
       (>= (ifaces current) 0)
       (car (list-ref avail-ifaces (ifaces current)))))

; Return mask for current interface choosed by user
(define (current-mask)
  (car (list-ref avail-masks (iface-mask current))))

; Startup procedure
(define (startup)
  ;; fill iface-mask widget with all possibles masks
  (ifaces rows (map cdr avail-ifaces))
  ;; load information about host
  (update-net-junior)
  ;; if global configuration isn't auto ask information about first interface
  (and (conf_type_is_manual)
       (positive? (ifaces count))
       (begin
         (ifaces current 0)
         (update-interface (current-interface))))
  )

; Fill nameservers widgets
(define (set_dns_servers servers-string)
  (let  ((servers-list (string-split servers-string #\newline)))
    (if (> (length servers-list) 0)
        (dns-1 text (list-ref servers-list 0)))
    (if (> (length servers-list) 1)
        (dns-2 text (list-ref servers-list 1)))
    (if (> (length servers-list) 2)
        (dns-3 text (list-ref servers-list 2)))))

;
; Read from backend
;

; Ask information about some interface
(define (update-interface name)
  (and (not-empty-string? name)
       (let ((cmd (woo-read-first (string-append "/net-junior" "/" name))))
         (iface-info text (woo-get-option cmd 'info))
         (w-button activity (woo-get-option cmd 'wireless #f))
         (let ((bootproto (woo-get-option cmd 'bootproto "unknown"))
               (disabled (woo-get-option cmd 'disabled #t)))
           (cond
             (  disabled
               (iface-conf current 3))
             (  (string=? bootproto "ipv4ll")
              (iface-conf current 2))
             (  (string=?  bootproto "static")
              (iface-conf current 0)   ; Statical IP address
              (iface-ip text (woo-get-option cmd 'ip))
              (iface-mask current (string-list-index (woo-get-option cmd 'mask "24") (map car avail-masks)))
              (iface-gw text (woo-get-option cmd 'default)))
             (  (string=? bootproto "dhcp")
              (iface-conf current 1)   ; use DHCP for this interface
              (dhcp-gw state (woo-get-option cmd 'dhcp-gw #t))         ; Set default route from DHCP parameters
              (dhcp-hostname state (woo-get-option cmd 'dhcp-hostname #f))   ; Set hostname from DHCP parameters
              (dhcp-dns state (woo-get-option cmd 'dhcp-dns #t))        ; Recive DNS servers from DHCP parameters
              (dhcp-ntp state (woo-get-option cmd 'dhcp-ntp #t))         ; Set default route from DHCP parameters
              (dhcp-conf rows (string-split (woo-get-option cmd 'dhcp_conf (_ "Interface doesn't yet configured by DHCP")) #\newline))) ; DHCP configuration
             )
           (and (not (string=? bootproto "dhcp")) ; Set default dhcp options
                (dhcp-hostname state #f)
                (dhcp-dns state #t)
                (dhcp-gw state #t)
                (dhcp-ntp state #t)
                (dhcp-conf rows `( ,(_ "Current DHCP configuration is unknown"))))
           (and (not (string=? bootproto "static"))
                (iface-mask current (string-list-index "24" (map car avail-masks))))
           ) ; end bootproto
         (iface_conf_type_changed))))

; Ask general information: FQDN, resolv-search, resolv-nameservers
(define (update-net-junior)
  (let ((data (woo-read-first "/net-junior")) (x #t))
    (hostname text (woo-get-option data 'hostname))
    (if (string= (woo-get-option data 'conf_type) "auto")
        (conf_auto state #t)   ; Auto (ipv4ll) configuration
        (conf_auto state #f)   ; Manual configuration
        )
    (set_dns_servers (woo-get-option data 'dns))
    (dns-search text (woo-get-option data 'search))
    )
  (conf_type_changed) ; Hide / show widgets for current selected information
  )

;
; Write to backend
;

; Interface configuration
(define (new-iface-conf)
  (cond
   ((iface_is_static)
    (list
     'disabled "no"
     'bootproto "static"
     'ip    (iface-ip text)
     'mask  (current-mask)
     'default (iface-gw text)))
   ((iface_is_dhcp)
    (list
     'disabled "no"
     'bootproto "dhcp"
     'dhcp_gw (dhcp-gw state)
     'dhcp_ntp (dhcp-ntp state)
     'dhcp_hostname (dhcp-hostname state)
     'dhcp_dns (dhcp-dns state)))
   ((iface_is_ip4ll)
    (list
     'disabled "no"
     'bootproto "ipv4ll"))
   (else
    (list
     'disabled "yes"))))

; Save selected inteface configuration
(define (write-interface path name)
  (and (not-empty-string? name)
       (woo-catch/message
        (thunk
         (apply woo-write/constraints
                (string-append path "/conf_iface") ; Interface name
                (append (new-iface-conf) `(iface ,name))))))) ; Interface configuration

; Configuration type: manual or auto (zeroconf avahi)
(define (write-conf-type path type)
  (woo-catch/message
   (thunk
    (woo-write/constraints
     (string-append path "/conf_type") ; Backend path
     'conf_type type
     ))))

(define (write-hostname path)
  (woo-catch/message
   (thunk
    (woo-write/constraints
     (string-append path "/conf_hostname") ; Backend path
     'hostname (hostname text)   ;hostname
     ))))

; Nameservers and search configuration
(define (write-resolv path)
  (woo-catch/message
   (thunk
    (woo-write/constraints
     (string-append path "/conf_resolv") ; Backend path
     'dns1 (dns-1 text)
     'dns2 (dns-2 text)
     'dns3 (dns-3 text)
     'search (dns-search text)))))

; XXX: woo cat with woo write? 
(define (restart-all-ifaces path)
  (splash-message (_ "Please wait some time while ethernet interfaces restarting"))
  (document:release)
  (woo-catch/message
    (thunk
      (woo-write
        (string-append path "/restart_all"))))
  (splash-message))

(define (restart-iface path iface)
  (splash-message (_ "Please wait some time while ethernet interface restarting"))
  (document:release)
  (woo-catch/message
    (thunk
      (woo-write
        (string-append path "/restart")
        'iface iface)))
  (splash-message))

; Commit changes
(define (commit-changes restart)
  (write-hostname junior-backend)
  (if (conf_type_is_auto)
      ; Auto (zeroconf-avahi)
      (begin
        (write-conf-type junior-backend "auto")
        (and restart (restart-all-ifaces junior-backend)))
      ; Manual (static or dhcp)
      (begin
        (write-conf-type junior-backend "manual")
        (write-resolv junior-backend)
        (write-interface junior-backend (current-interface))
        (and restart (restart-iface junior-backend (current-interface))))))
